@@ -12,7 +12,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: logfetch.c 7139 2012-08-01 11:40:10Z storner $";
+static char rcsid[] = "$Id: logfetch.c 7630 2015-04-17 13:52:05Z jccleaver $";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -36,12 +36,21 @@ static char rcsid[] = "$Id: logfetch.c 7139 2012-08-01 11:40:10Z storner $";
 
 #include "libxymon.h"
 
+/* Set via xgetenv */
+static char skiptxt[512];
+static char curpostxt[512];
+
 /* Is it ok for these to be hardcoded ? */
 #define MAXCHECK   102400   /* When starting, dont look at more than 100 KB of data */
 #define MAXMINUTES 30
-#define POSCOUNT ((MAXMINUTES / 5) + 1)
+#define POSCOUNT ((MAXMINUTES / 5) + 1)		/* 0 = current run */
+#define DEFAULTSCROLLBACK (POSCOUNT - 1)	/* How far back to begin processing data, in runs */
 #define LINES_AROUND_TRIGGER 5
 
+/* Default = use default */
+int scrollback = -1;
+
+
 typedef enum { C_NONE, C_LOG, C_FILE, C_DIR, C_COUNT } checktype_t;
 
 typedef struct logdef_t {
@@ -56,6 +65,10 @@ typedef struct logdef_t {
 	int triggercount;
 	char **ignore;
 	int ignorecount;
+	char **deltacountpatterns;
+	char **deltacountnames;
+	int deltacountcount;
+	int *deltacountcounts;
 } logdef_t;
 
 typedef struct filedef_t {
@@ -102,27 +115,62 @@ FILE *fileopen(char *filename, int *err)
 	return fd;
 }
 
+/*
+ * A wrapper for fgets() which eats embedded 0x00 characters in the stream.
+ */
+char *fgets_nonull(char *buf, size_t size, FILE *stream) {
+	char *in, *out, *end;
+
+	if (fgets(buf, size - 1, stream) == NULL) 
+	        return NULL;
+
+	end = memchr(buf, '\n', size - 1); 
+
+	if (end == NULL) 
+	        end = buf + (size - 1); 
+	else 
+	        end++;
+
+	for (in = out = buf; in < end; in++) {
+	        if (*in != '\0') 
+		       *out++ = *in;
+	}
+
+	*out = '\0';
+
+	return buf;
+}
+
 
 char *logdata(char *filename, logdef_t *logdef)
 {
-	static char *buf = NULL;
-	char *startpos, *fillpos, *triggerstartpos, *triggerendpos;
+	static char *buf, *replacement = NULL;
+	char *startpos, *fillpos, *triggerstartpos, *triggerendpos, *curpos = NULL;
 	FILE *fd;
 	struct stat st;
 	size_t bytesread, bytesleft;
 	int openerr, i, status, triggerlinecount, done;
 	char *linepos[2*LINES_AROUND_TRIGGER+1];
 	int lpidx;
-	regex_t *ignexpr = NULL, *trigexpr = NULL;
+	size_t byteslast, bytestocurrent, bytesin = 0;
+	regex_t *deltaexpr = NULL, *ignexpr = NULL, *trigexpr = NULL;
 #ifdef _LARGEFILE_SOURCE
 	off_t bufsz;
 #else
 	long bufsz;
 #endif
 
+	char *(*triggerptrs)[2] = NULL;
+	unsigned int triggerptrs_count = 0;
+
+	dbgprintf("logfetch: -> logdata (%s)\n", filename);
+
 	if (buf) free(buf);
 	buf = NULL;
 
+	if (replacement) free(replacement);
+	replacement = NULL;
+
 	fd = fileopen(filename, &openerr);
 	if (fd == NULL) {
 		buf = (char *)malloc(1024 + strlen(filename));
@@ -136,43 +184,56 @@ char *logdata(char *filename, logdef_t *logdef)
 	 * from one run to the next.
 	 */
 	fstat(fileno(fd), &st);
-	if ((st.st_size < logdef->lastpos[0]) || (st.st_size < logdef->lastpos[6])) {
+	if ((st.st_size < logdef->lastpos[0]) || (st.st_size < logdef->lastpos[POSCOUNT-1])) {
 		/*
 		 * Logfile shrank - probably it was rotated.
 		 * Start from beginning of file.
 		 */
-		for (i=0; (i < 7); i++) logdef->lastpos[i] = 0;
+		errprintf("logfetch: File %s shrank from >=%zu to %zu bytes in size. Probably rotated; clearing position state\n", filename, logdef->lastpos[POSCOUNT-1], st.st_size);
+		for (i=0; (i < POSCOUNT); i++) logdef->lastpos[i] = 0;
 	}
 
-	/* Go to the position we were at 6 times ago (corresponds to 30 minutes) */
+	/* Go to the position we were at scrollback times ago (default corresponds to 6 -- 30 minutes when 5m per run) */
 #ifdef _LARGEFILE_SOURCE
-	fseeko(fd, logdef->lastpos[6], SEEK_SET);
+	fseeko(fd, logdef->lastpos[scrollback], SEEK_SET);
 	bufsz = st.st_size - ftello(fd);
 	if (bufsz > MAXCHECK) {
 		/*
 		 * Too much data for us. We have to skip some of the old data.
 		 */
-		logdef->lastpos[6] = st.st_size - MAXCHECK;
-		fseeko(fd, logdef->lastpos[6], SEEK_SET);
+		errprintf("logfetch: %s delta %zu bytes exceeds max buffer size %zu; skipping some data\n", filename, bufsz, MAXCHECK);
+		logdef->lastpos[scrollback] = st.st_size - MAXCHECK;
+		fseeko(fd, logdef->lastpos[scrollback], SEEK_SET);
 		bufsz = st.st_size - ftello(fd);
 	}
 #else
-	fseek(fd, logdef->lastpos[6], SEEK_SET);
+	fseek(fd, logdef->lastpos[scrollback], SEEK_SET);
 	bufsz = st.st_size - ftell(fd);
 	if (bufsz > MAXCHECK) {
 		/*
 		 * Too much data for us. We have to skip some of the old data.
 		 */
-		logdef->lastpos[6] = st.st_size - MAXCHECK;
-		fseek(fd, logdef->lastpos[6], SEEK_SET);
+		errprintf("logfetch: %s delta %zu bytes exceeds max buffer size %zu; skipping some data\n", filename, bufsz, MAXCHECK);
+		logdef->lastpos[scrollback] = st.st_size - MAXCHECK;
+		fseek(fd, logdef->lastpos[scrollback], SEEK_SET);
 		bufsz = st.st_size - ftell(fd);
 	}
 #endif
 
+	/* Calculate delta between scrollback (where going to start looking at data) and end of the most recent run.
+	 * This is where we place our "CURRENT" marker */
+	byteslast = logdef->lastpos[scrollback];
+	/* If lastpos[0] is 0, then all of the positions are 0 because we rotated above */
+	bytestocurrent = logdef->lastpos[0] - byteslast;
+
 	/* Shift position markers one down for the next round */
-	for (i=6; (i > 0); i--) logdef->lastpos[i] = logdef->lastpos[i-1];
+	/* lastpos[1] is the previous end location, lastpos[0] will be the end of this run */
+	for (i=POSCOUNT-1; (i > 0); i--) logdef->lastpos[i] = logdef->lastpos[i-1];
 	logdef->lastpos[0] = st.st_size;
 
+	dbgprintf("logfetch: Current size (ending): %zu bytes. Last end was %zu. Looking %d spots before that, which is %zu. bytestocurrent: %zu\n",
+		logdef->lastpos[0], logdef->lastpos[1], scrollback, byteslast, bytestocurrent);
+
 	/*
 	 * Get our read buffer.
 	 *
@@ -182,7 +243,7 @@ char *logdata(char *filename, logdef_t *logdef)
 	 *     it still hasnt reached end-of-file status.
 	 *     At least, on some platforms (Solaris, FreeBSD).
 	 */
-	bufsz += 1023;
+	bufsz += 1023 + strlen(curpostxt);
 	startpos = buf = (char *)malloc(bufsz + 1);
 	if (buf == NULL) {
 		/* Couldnt allocate the buffer */
@@ -190,21 +251,52 @@ char *logdata(char *filename, logdef_t *logdef)
 	}
 
 	/* Compile the regex patterns */
+	if (logdef->deltacountcount) {
+		int i, realcount = 0;
+
+		deltaexpr = (regex_t *) malloc(logdef->deltacountcount * sizeof(regex_t));
+		for (i=0; (i < logdef->deltacountcount); i++) {
+			dbgprintf(" - compiling DELTACOUNT regex: %s\n", logdef->deltacountpatterns[i]);
+			status = regcomp(&deltaexpr[realcount++], logdef->deltacountpatterns[i], REG_EXTENDED|REG_ICASE|REG_NOSUB);
+			if (status != 0) {
+				char regbuf[1000];
+				regerror(status, &deltaexpr[--realcount], regbuf, sizeof(regbuf));	/* re-decrement realcount here */
+				errprintf("logfetch: could not compile deltacount regex '%s': %s\n", logdef->deltacountpatterns[i], regbuf);
+				logdef->deltacountpatterns[i] = logdef->deltacountnames[i] = NULL;
+			}
+		}
+		logdef->deltacountcount = realcount;
+		logdef->deltacountcounts = (int *)calloc(logdef->deltacountcount, sizeof(int));	// initialize the equivalent count array with 0's
+	}
 	if (logdef->ignorecount) {
-		int i;
+               int i, realcount = 0;
 		ignexpr = (regex_t *) malloc(logdef->ignorecount * sizeof(regex_t));
 		for (i=0; (i < logdef->ignorecount); i++) {
-			status = regcomp(&ignexpr[i], logdef->ignore[i], REG_EXTENDED|REG_ICASE|REG_NOSUB);
-			if (status != 0) logdef->ignore[i] = NULL;
+			dbgprintf(" - compiling IGNORE regex: %s\n", logdef->ignore[i]);
+			status = regcomp(&ignexpr[realcount++], logdef->ignore[i], REG_EXTENDED|REG_ICASE|REG_NOSUB);
+			if (status != 0) {
+				char regbuf[1000];
+				regerror(status, &ignexpr[--realcount], regbuf, sizeof(regbuf));	/* re-decrement realcount here */
+				errprintf("logfetch: could not compile ignore regex '%s': %s\n", logdef->ignore[i], regbuf);
+				logdef->ignore[i] = NULL;
+			}
 		}
+		logdef->ignorecount = realcount;
 	}
 	if (logdef->triggercount) {
-		int i;
+		int i, realcount = 0;
 		trigexpr = (regex_t *) malloc(logdef->triggercount * sizeof(regex_t));
 		for (i=0; (i < logdef->triggercount); i++) {
-			status = regcomp(&trigexpr[i], logdef->trigger[i], REG_EXTENDED|REG_ICASE|REG_NOSUB);
-			if (status != 0) logdef->trigger[i] = NULL;
+			dbgprintf(" - compiling TRIGGER regex: %s\n", logdef->trigger[i]);
+			status = regcomp(&trigexpr[realcount++], logdef->trigger[i], REG_EXTENDED|REG_ICASE|REG_NOSUB);
+			if (status != 0) {
+				char regbuf[1000];
+				regerror(status, &trigexpr[--realcount], regbuf, sizeof(regbuf));	/* re-decrement realcount here */
+				errprintf("logfetch: could not compile trigger regex '%s': %s\n", logdef->trigger[i], regbuf);
+				logdef->trigger[i] = NULL;
+			}
 		}
+		logdef->triggercount = realcount;
 	}
 	triggerstartpos = triggerendpos = NULL;
 	triggerlinecount = 0;
@@ -218,7 +310,24 @@ char *logdata(char *filename, logdef_t *logdef)
 	fillpos = buf;
 	bytesleft = bufsz;
 	done = 0;
-	while (!ferror(fd) && (bytesleft > 0) && !done && (fgets(fillpos, bytesleft, fd) != NULL)) {
+	while (!ferror(fd) && (bytesleft > 0) && !done && (fgets_nonull(fillpos, bytesleft, fd) != NULL)) {
+		int force_trigger = 0;
+
+		/* Mark where we left off */
+		bytesin += strlen(fillpos);
+		if (!curpos && (bytesin >= bytestocurrent)) {
+			char *t;
+
+			dbgprintf(" - Last position was %u, found curpos location at %u in current buffer.\n", logdef->lastpos[1], bytesin);
+			t = strdup(fillpos);	/* need shuffle about to insert before this line */
+			strncpy(fillpos, curpostxt, strlen(curpostxt));		/* add in the CURRENT + \n */
+			strncpy(fillpos+strlen(curpostxt), t, strlen(t));	/* add in whatever this line originally was */
+			*(fillpos+strlen(curpostxt)+strlen(t)) = '\0';		/* and terminate it */
+			xfree(t);						/* free temp */
+			curpos = fillpos;					/* leave curpos to the beginning of the CURRENT flag */
+			force_trigger = 1;					/* We'll force this to be saved as a trigger later on, so it always gets printed */
+		}
+
 		if (*fillpos == '\0') {
 			/*
 			 * fgets() can return an empty buffer without flagging
@@ -227,29 +336,55 @@ char *logdata(char *filename, logdef_t *logdef)
 			 * terminating \0 byte, but if it does then we will
 			 * catch it here.
 			 */
+			dbgprintf(" - empty buffer returned; assuming eof\n");
 			done = 1;
 			continue;
 		}
 
+		/* Begin counting lines once we've reached the end of the last run */
+		if (curpos && logdef->deltacountcount) {
+			int i, match = 0;
+
+			for (i=0; (i < logdef->deltacountcount); i++) {
+				match = (regexec(&deltaexpr[i], fillpos, 0, NULL, 0) == 0);
+				if (match) { logdef->deltacountcounts[i]++; dbgprintf(" - line matched deltacount %d: %s", i, fillpos); } // fgets stores the newline in
+			}
+		}
+
 		/* Check ignore pattern */
 		if (logdef->ignorecount) {
 			int i, match = 0;
 
 			for (i=0; ((i < logdef->ignorecount) && !match); i++) {
 				match = (regexec(&ignexpr[i], fillpos, 0, NULL, 0) == 0);
+				if (match) dbgprintf(" - line matched ignore %d: %s", i, fillpos); // fgets stores the newline in
 			}
 
-			if (match) continue;
+			if (force_trigger) {
+				/* Oops. We actually wanted this line poked through */
+				/* At the moment, we're only entering this state when we've added 'CURRENT\n' to the existing line */
+				/* Since we're guaranteeing to downstream users that we're ignoring this, just truncate the line */
+				/* to the first newline. If we start using force_trigger for other purposes, we might have to change */
+				/* the logic here. */
+				char *eoln;
+
+				eoln = strchr(fillpos, '\n'); if (eoln) *++eoln = '\0';	// still need the final newline
+			}
+			else if (match) continue;
 		}
 
 		linepos[lpidx] = fillpos;
 
 		/* See if this is a trigger line */
-		if (logdef->triggercount) {
+		if (force_trigger || logdef->triggercount) {
 			int i, match = 0;
 
-			for (i=0; ((i < logdef->ignorecount) && !match); i++) {
+			if (force_trigger) { match = 1; dbgprintf(" - line forced as a trigger: %s", fillpos); }
+			else {
+			    for (i=0; ((i < logdef->triggercount) && !match); i++) {
 				match = (regexec(&trigexpr[i], fillpos, 0, NULL, 0) == 0);
+				if (match) dbgprintf(" - line matched trigger %d: %s", i, fillpos); // fgets stores the newline in
+			    }
 			}
 
 			if (match) {
@@ -259,6 +394,30 @@ char *logdata(char *filename, logdef_t *logdef)
 				if (sidx < 0) sidx += (2*LINES_AROUND_TRIGGER + 1);
 				triggerstartpos = linepos[sidx]; if (!triggerstartpos) triggerstartpos = buf;
 				triggerlinecount = LINES_AROUND_TRIGGER;
+
+				if (triggerptrs == NULL || (triggerptrs_count > 0 && triggerptrs[triggerptrs_count - 1][1] != NULL)) {
+					dbgprintf(" - %s trigger line encountered; preparing trigger START & END positioning store\n", ((triggerptrs_count == 0) ? "first" : "additional")); 
+
+					/* Create or resize the trigger pointer array to contain another pair of anchors */
+   					triggerptrs = realloc(triggerptrs, (sizeof(char *) * 2) * (++triggerptrs_count));
+					if (triggerptrs == NULL) return "Out of memory";
+
+					/* Save the current triggerstartpos as our first anchor in the pair */
+					triggerptrs[triggerptrs_count - 1][0] = triggerstartpos;
+					triggerptrs[triggerptrs_count - 1][1] = NULL;
+
+					if (triggerptrs_count > 1 && (triggerstartpos <= triggerptrs[triggerptrs_count - 2][1])) {
+						/* Whoops! This trigger's LINES_AROUND_TRIGGER bleeds into the prior's LINES_AROUND_TRIGGER */
+						triggerptrs[triggerptrs_count - 1][0] = triggerptrs[triggerptrs_count - 2][1];
+						dbgprintf("Current trigger START (w/ prepended LINES_AROUND_TRIGGER) would overlap with prior trigger's END. Adjusting.\n");
+					}
+
+					dbgprintf(" - new trigger anchor START position set\n");
+		               } 
+		               else {
+					/* Do nothing. Merge the two trigger lines into a single start and end pair by extending the existing */
+					dbgprintf("Additional trigger line encountered. Previous trigger START has no set END yet. Compressing anchors.\n");
+				}
 			}
 		}
 
@@ -271,9 +430,23 @@ char *logdata(char *filename, logdef_t *logdef)
 		if (triggerlinecount) {
 			triggerlinecount--;
 			triggerendpos = fillpos;
+
+			if (triggerlinecount == 0) {
+				/* Terminate the current trigger anchor pair by aligning the end pointer */
+				dbgprintf(" - trigger END position set\n");
+				triggerptrs[triggerptrs_count - 1][1] = triggerendpos;
+			}
 		}
 
 		bytesleft = (bufsz - (fillpos - buf));
+		// dbgprintf(" -- bytesleft: %zu\n", bytesleft);
+	}
+
+	if (triggerptrs != NULL) {
+	        dbgprintf("Marked %i pairs of START and END anchors for consideration.\n", triggerptrs_count);
+
+		/* Ensure that a premature EOF before the last trigger end postion doesn't blow up */
+		if (triggerptrs[triggerptrs_count - 1][1] == NULL) triggerptrs[triggerptrs_count -1][1] = fillpos;
 	}
 
 	/* Was there an error reading the file? */
@@ -288,41 +461,95 @@ char *logdata(char *filename, logdef_t *logdef)
 	*(buf + bytesread) = '\0';
 
 	if (bytesread > logdef->maxbytes) {
-		char *skiptxt = "<...SKIPPED...>\n";
 
-		/* FIXME: Must make sure to only pass complete lines back to the server */
-		if (triggerstartpos) {
-			/* Skip the beginning of the data up until the trigger was found */
-			startpos = triggerstartpos;
-			if ((startpos - strlen(skiptxt)) >= buf) {
-				startpos -= strlen(skiptxt);
-				memcpy(startpos, skiptxt, strlen(skiptxt));
-			}
-			bytesread = (fillpos - startpos);
+	        if (triggerptrs != NULL) {
+		       size_t triggerbytes, nontriggerbytes, skiptxtbytes, lasttriggeroffset;
+		       char *pos, *lasttriggerptr;
+		       size_t size;
 
-			/*
-			 * If it's still too big, show some lines after the trigger, and
-			 * then skip until it will fit.
-			 */
-			if (bytesread > logdef->maxbytes) {
-				size_t bytesleft;
+		       /* Sum the number of bytes required to hold all the trigger content (start -> end anchors) */
+		       for (i = 0, triggerbytes = 0, skiptxtbytes = 0; i < triggerptrs_count; i++) {
+		           triggerbytes += strlen(triggerptrs[i][0]) - strlen(triggerptrs[i][1]);
+		           skiptxtbytes += strlen(skiptxt) * 2;
+		       }
+	  
+		       /* Find the remaining bytes allowed for non-trigger content (and prevent size_t underflow wrap ) */
+			nontriggerbytes = (logdef->maxbytes < triggerbytes) ? 0 : (logdef->maxbytes - triggerbytes);
+			lasttriggerptr = triggerptrs[triggerptrs_count - 1][1];
+			lasttriggeroffset = (fillpos - lasttriggerptr);
 
-				bytesleft = logdef->maxbytes - (triggerendpos - startpos);
-				if (bytesleft > 0) {
-					char *skipend;
+		       dbgprintf("Found %zu bytes of trigger data, %zu bytes available space for non-trigger data; last trigger ended %zu bytes from end. Max bytes allowed is %zu.\n", triggerbytes, nontriggerbytes, lasttriggeroffset, logdef->maxbytes);
 
-					skipend = fillpos - bytesleft;
-					memmove(triggerendpos, skipend, bytesleft);
-					*(triggerendpos + bytesleft) = '\0';
+		       /* Allocate a new buffer, reduced to what we actually can hold */
+		       replacement = malloc(sizeof(char) * (triggerbytes + skiptxtbytes + nontriggerbytes + 1));
+		       if (replacement == NULL) return "Out of memory";
 
-					if (bytesleft >= strlen(skiptxt)) 
-						memcpy(triggerendpos, skiptxt, strlen(skiptxt));
-					bytesread = (triggerendpos - startpos) + bytesleft;
+		       dbgprintf("Staging replacement buffer, %zu bytes.\n", (triggerbytes + skiptxtbytes + nontriggerbytes));
+
+		       /* Iterate each trigger anchor pair, copying into the replacement */
+		       for (i = 0, pos = replacement; i < triggerptrs_count; i++) {
+		               dbgprintf("Copying buffer content for trigger %i.\n", (i + 1));
+
+		               strncpy(pos, skiptxt, strlen(skiptxt));
+		               pos += strlen(skiptxt);
+
+		               size = strlen(triggerptrs[i][0]) - strlen(triggerptrs[i][1]);
+		               strncpy(pos, triggerptrs[i][0], size);
+		               pos += size;
+		       }
+
+		       /* At this point, all the required trigger lines are present */
+			*(pos) = '\0';
+
+		       if (nontriggerbytes > 0) {
+				char *finalstartptr;
+
+				/* Append non-trigger, up to the allowed byte size remaining, of remaining log content, starting backwards */
+
+				/* Figure out where to start copying from */
+				finalstartptr = (fillpos - nontriggerbytes);
+				if (finalstartptr < lasttriggerptr) {
+					/* 
+					 * FIXME: We could have included inter-trigger data here too.
+					 * At the moment, duplicate lines are the worse of two evils, so just start from there.
+					 */
+					finalstartptr = lasttriggerptr;
+					nontriggerbytes = (fillpos - finalstartptr);
+					dbgprintf("logfetch: More space was available than between last trigger and eof; reducing to %zu bytes\n", nontriggerbytes);
 				}
-			}
-		}
+
+				/* We're already skipping content; so don't send a partial line. Be sure to decrement the subsequent length we feed to strncpy */
+				while (*(finalstartptr-1) != '\n') {
+					finalstartptr += 1; nontriggerbytes -= 1;
+				}
+
+				dbgprintf("logfetch: Delta from end of final trigger to beginning of final section: %zu bytes\n", (finalstartptr - lasttriggerptr) );
+
+				if (finalstartptr > lasttriggerptr) {
+					/* Add the final skip for completeness */
+					strncpy(pos, skiptxt, strlen(skiptxt));
+					pos += strlen(skiptxt);
+				}
+
+				/* And copy the the rest of the original buffer content */
+				dbgprintf("Copying %zu final bytes of non-trigger content\n", nontriggerbytes);
+				strncpy(pos, finalstartptr, nontriggerbytes);
+				*(pos + nontriggerbytes) = '\0';	/* re-terminate */
+		       }
+
+		       /* Prune out the last line to prevent sending a partial */
+		       if (*(pos = &replacement[strlen(replacement) - 1]) != '\n') {
+		               while (*pos != '\n') {
+		                       pos -= 1;
+		               }
+		               *(++pos) = '\0';
+		       }
+
+		       startpos = replacement;
+		       bytesread = strlen(startpos);          
+	        }
 		else {
-			/* Just drop what is too much */
+			/* Just drop what is too much -- buf+bytesread was terminated above */
 			startpos += (bytesread - logdef->maxbytes);
 			memcpy(startpos, skiptxt, strlen(skiptxt));
 			bytesread = logdef->maxbytes;
@@ -347,6 +574,13 @@ cleanup:
 	{
 		int i;
 
+		if (logdef->deltacountcount) {
+			for (i=0; (i < logdef->deltacountcount); i++) {
+				if (logdef->deltacountpatterns[i]) regfree(&deltaexpr[i]);
+			}
+			xfree(deltaexpr);
+		}
+
 		if (logdef->ignorecount) {
 			for (i=0; (i < logdef->ignorecount); i++) {
 				if (logdef->ignore[i]) regfree(&ignexpr[i]);
@@ -360,8 +594,11 @@ cleanup:
 			}
 			xfree(trigexpr);
 		}
+
+		if (triggerptrs) xfree(triggerptrs);
 	}
 
+	dbgprintf("logfetch: <- logdata (%s)\n", filename);
 	return startpos;
 }
 
@@ -580,6 +817,7 @@ int loadconfig(char *cfgfn)
 	 *    log:filename:maxbytes
 	 *    ignore ignore-regexp (optional)
 	 *    trigger trigger-regexp (optional)
+	 *    deltacount label deltacount-regexp (optional)
 	 *
 	 *    file:filename
 	 */
@@ -812,6 +1050,61 @@ int loadconfig(char *cfgfn)
 					currcfg->check.logcheck.trigger[currcfg->check.logcheck.triggercount-1] = strdup(p);
 				}
 			}
+			else if (strncmp(bol, "deltacount ", 11) == 0) {
+				char *p; 
+
+				p = bol + 11; p += strspn(p, " \t");
+
+				if (firstpipeitem) {
+					/* Fill in this trigger expression on all items in this pipe set */
+					checkdef_t *walk = currcfg;
+					char *name, *ptn = NULL;
+
+					name = strtok(p, " :");
+					if (name) ptn = strtok(NULL, "\n");
+
+					if (name && ptn) {
+					    do {
+						walk->check.logcheck.deltacountcount++;
+
+						walk->check.logcheck.deltacountnames = 
+							realloc(walk->check.logcheck.deltacountnames,
+								(walk->check.logcheck.deltacountcount)*sizeof(char *));
+
+						walk->check.logcheck.deltacountpatterns = 
+							realloc(walk->check.logcheck.deltacountpatterns,
+								(walk->check.logcheck.deltacountcount)*sizeof(char *));
+
+						walk->check.logcheck.deltacountnames[walk->check.logcheck.deltacountcount-1] = strdup(name);
+						walk->check.logcheck.deltacountpatterns[walk->check.logcheck.deltacountcount-1] = strdup(ptn);
+
+						walk = walk->next;
+
+					    } while (walk && (walk != firstpipeitem->next));
+					}
+				}
+				else {
+					char *name, *ptn = NULL;
+
+					name = strtok(p, " :");
+					if (name) ptn = strtok(NULL, "\n");
+
+					if (name && ptn) {
+						currcfg->check.logcheck.deltacountcount++;
+
+						currcfg->check.logcheck.deltacountnames = 
+							realloc(currcfg->check.logcheck.deltacountnames,
+								(currcfg->check.logcheck.deltacountcount)*sizeof(char *));
+
+						currcfg->check.logcheck.deltacountpatterns = 
+							realloc(currcfg->check.logcheck.deltacountpatterns,
+								(currcfg->check.logcheck.deltacountcount)*sizeof(char *));
+
+						currcfg->check.logcheck.deltacountnames[currcfg->check.logcheck.deltacountcount-1] = strdup(name);
+						currcfg->check.logcheck.deltacountpatterns[currcfg->check.logcheck.deltacountcount-1] = strdup(ptn);
+					}
+				}
+			}
 		}
 		else if (currcfg && (currcfg->checktype == C_FILE)) {
 			/* Nothing */
@@ -927,7 +1220,12 @@ int main(int argc, char *argv[])
 #endif
 
 	for (i=1; (i<argc); i++) {
-		if (strcmp(argv[i], "--clock") == 0) {
+		if (strncmp(argv[i], "--debug", 7) == 0) {
+			char *delim = strchr(argv[i], '=');
+			debug = 1;
+			if (delim) set_debugfile(delim+1, 0);
+		}
+		else if (strcmp(argv[i], "--clock") == 0) {
 			struct timeval tv;
 			struct timezone tz;
 			struct tm *tm;
@@ -950,16 +1248,54 @@ int main(int argc, char *argv[])
 			printf("%s\n", timestr);
 			return 0;
 		}
-		else if (i == 1) cfgfn = argv[i];
-		else if (i == 2) statfn = argv[i];
+		else if ((*(argv[i]) == '-') && (strlen(argv[i]) > 1)) {
+			fprintf(stderr, "Unknown option %s\n", argv[i]);
+		}
+		else {
+			/* Not an option -- should have two arguments left: our config and status file */
+			if (cfgfn == NULL) cfgfn = argv[i];
+			else if (statfn == NULL) statfn = argv[i];
+			else fprintf(stderr, "Unknown argument '%s'\n", argv[i]);
+		}
 	}
 
-	if ((cfgfn == NULL) || (statfn == NULL)) return 1;
+
+	if (scrollback == -1) {
+		char *p;
+
+		p = getenv("LOGFETCHSCROLLBACK");
+		if (!p) p = getenv("LOGFETCH_SCROLLBACK"); /* compat */
+		if (!p || (strlen(p) == 0) ) scrollback = DEFAULTSCROLLBACK;
+		else {
+			int scroll;
+			scroll = atoi(p);
+
+			/* Don't use DEFAULTSCROLLBACK here in case we change or lower the default in the future */
+			if ((scroll < 0) || (scroll > (POSCOUNT-1)) ) {
+				errprintf("Scrollback requested (%d) is greater than max saved (%d)\n", scroll, (POSCOUNT - 1) );
+				scrollback = DEFAULTSCROLLBACK;
+			}
+			else {
+				dbgprintf("logfetch: setting scrollback to %d\n", scroll);
+				scrollback = scroll;
+			}
+		}
+	}
+
+
+	if ((cfgfn == NULL) || (statfn == NULL)) {
+		fprintf(stderr, "Missing config or status file arguments\n");
+		return 1;
+	}
 
 	if (loadconfig(cfgfn) != 0) return 1;
 	loadlogstatus(statfn);
 
+	snprintf(skiptxt, sizeof(skiptxt), "%s\n", xgetenv("LOGFETCHSKIPTEXT"));
+	snprintf(curpostxt, sizeof(curpostxt), "%s\n", xgetenv("LOGFETCHCURRENTTEXT"));
+
 	for (walk = checklist; (walk); walk = walk->next) {
+		int idx;
 		char *data;
 		checkdef_t *fwalk;
 
@@ -969,6 +1305,11 @@ int main(int argc, char *argv[])
 			fprintf(stdout, "[msgs:%s]\n", walk->filename);
 			fprintf(stdout, "%s\n", data);
 
+			if (walk->check.logcheck.deltacountcount) fprintf(stdout, "[deltacount:%s]\n", walk->filename);
+			for (idx = 0; (idx < walk->check.logcheck.deltacountcount); idx++) {
+				fprintf(stdout, "%s: %d\n", walk->check.logcheck.deltacountnames[idx], walk->check.logcheck.deltacountcounts[idx]);
+			}
+
 			/* See if there's a special "file:" entry for this logfile */
 			for (fwalk = checklist; (fwalk && ((fwalk->checktype != C_FILE) || (strcmp(fwalk->filename, walk->filename) != 0))); fwalk = fwalk->next) ;
 			if (fwalk == NULL) {
