@@ -9,7 +9,7 @@
 #                                                                            #
 #----------------------------------------------------------------------------#
 #
-# $Id: xymonclient-linux.sh 7362 2014-01-23 20:42:41Z storner $
+# $Id: xymonclient-linux.sh 7618 2015-03-30 03:27:16Z jccleaver $
 
 echo "[date]"
 date
@@ -30,7 +30,7 @@ elif [ -f /etc/debian_version ]; then
 	echo -n "Debian "
 	cat /etc/debian_version
 elif [ -f /etc/S?SE-release ]; then
-	egrep -i "^suse|^opensuse" /etc/S?SE-release
+	cat /etc/S?SE-release
 elif [ -f /etc/slackware-version ]; then
 	cat /etc/slackware-version
 elif [ -f /etc/mandrake-release ]; then
@@ -68,13 +68,13 @@ echo "[netstat]"
 netstat -s
 echo "[ports]"
 # Bug in RedHat's netstat spews annoying error messages. 
-netstat -ant 2>/dev/null
+netstat -antu 2>/dev/null
 echo "[ifstat]"
 /sbin/ifconfig 2>/dev/null
 # Report mdstat data if it exists
 if test -r /proc/mdstat; then echo "[mdstat]"; cat /proc/mdstat; fi
 echo "[ps]"
-ps -Aww -o pid,ppid,user,start,state,pri,pcpu,time:12,pmem,rsz:10,vsz:10,cmd
+ps -Aww f -o pid,ppid,user,start,state,pri,pcpu,time:12,pmem,rsz:10,vsz:10,cmd
 
 # $TOP must be set, the install utility should do that for us if it exists.
 if test "$TOP" != ""
@@ -82,6 +82,7 @@ then
     if test -x "$TOP"
     then
         echo "[top]"
+	export CPULOOP ; CPULOOP=1 ;
 	$TOP -b -n 1 
 	# Some top versions do not finish off the last line of output
 	echo ""
