@@ -12,7 +12,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: loadalerts.c 7359 2014-01-21 16:09:01Z storner $";
+static char rcsid[] = "$Id: loadalerts.c 7663 2015-05-21 14:28:50Z jccleaver $";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -182,6 +182,7 @@ static void free_criteria(criteria_t *crit)
 	if (crit->exgroupspec)   xfree(crit->exgroupspec);
 	if (crit->exgroupspecre) pcre_free(crit->exgroupspecre);
 	if (crit->timespec)      xfree(crit->timespec);
+	if (crit->extimespec)    xfree(crit->extimespec);
 }
 
 int load_alertconfig(char *configfn, int defcolors, int defaultinterval)
@@ -476,6 +477,16 @@ int load_alertconfig(char *configfn, int defcolors, int defaultinterval)
 				crit->timespec = strdup(val);
 				firsttoken = 0;
 			}
+			else if ((strncasecmp(p, "EXTIME=", 7) == 0) || (strncasecmp(p, "EXTIMES=", 8) == 0)) {
+				char *val;
+				criteria_t *crit;
+
+				if (firsttoken) { flush_rule(currule); currule = NULL; currcp = NULL; pstate = P_NONE; }
+				val = strchr(p, '=')+1;
+				crit = setup_criteria(&currule, &currcp);
+				crit->extimespec = strdup(val);
+				firsttoken = 0;
+			}
 			else if (strncasecmp(p, "DURATION", 8) == 0) {
 				criteria_t *crit;
 
@@ -729,6 +740,7 @@ static void dump_criteria(criteria_t *crit, int isrecip)
 	}
 
 	if (crit->timespec) printf("TIME=%s ", crit->timespec);
+	if (crit->extimespec) printf("EXTIME=%s ", crit->extimespec);
 	if (crit->minduration) printf("DURATION>%d ", (crit->minduration / 60));
 	if (crit->maxduration) printf("DURATION<%d ", (crit->maxduration / 60));
 	if (isrecip) {
@@ -788,19 +800,26 @@ static int criteriamatch(activealerts_t *alert, criteria_t *crit, criteria_t *ru
 {
 	/*
 	 * See if the "crit" matches the "alert".
-	 * Match on pagespec, dgspec, hostspec, svcspec, classspec, groupspec, colors, timespec, minduration, maxduration, sendrecovered
+	 * Match on pagespec, dgspec, hostspec, svcspec, classspec, groupspec, colors, timespec, extimespec, minduration, maxduration, sendrecovered
 	 */
 
 	static char *pgnames = NULL;
+	char *dgname = NULL;
 	int pgmatchres, pgexclres;
 	time_t duration = (getcurrenttime(NULL) - alert->eventstart);
 	int result, cfid = 0;
 	char *pgtok, *cfline = NULL;
 	void *hinfo = hostinfo(alert->hostname);
 
+	if (!hinfo) {
+		dbgprintf("Checking criteria for host '%s', which is not defined\n", alert->hostname);
+		hinfo = localhostinfo(alert->hostname);
+	};
+
 	/* The top-level page needs a name - cannot match against an empty string */
 	if (pgnames) xfree(pgnames);
 	pgnames = strdup((*alert->location == '\0') ? "/" : alert->location);
+	dgname = textornull(xmh_item(hinfo, XMH_DGNAME));
 
 	if (crit) { cfid = crit->cfid; cfline = crit->cfline; }
 	if (!cfid && rulecrit) cfid = rulecrit->cfid;
@@ -808,7 +827,7 @@ static int criteriamatch(activealerts_t *alert, criteria_t *crit, criteria_t *ru
 	if (!cfline) cfline = "<undefined>";
 
 	traceprintf("Matching host:service:dgroup:page '%s:%s:%s:%s' against rule line %d\n",
-			alert->hostname, alert->testname, xmh_item(hinfo, XMH_DGNAME), alert->location, cfid);
+			alert->hostname, alert->testname, dgname, alert->location, cfid);
 
 	if (alert->state == A_PAGING) {
 		/* Check max-duration now - it's fast and easy. */
@@ -904,11 +923,11 @@ static int criteriamatch(activealerts_t *alert, criteria_t *crit, criteria_t *ru
 		return 0;
 	}
 
-	if (crit && crit->dgspec && !namematch(xmh_item(hinfo, XMH_DGNAME), crit->dgspec, crit->dgspecre)) { 
+	if (crit && crit->dgspec && !namematch(dgname, crit->dgspec, crit->dgspecre)) { 
 		traceprintf("Failed '%s' (displaygroup not in include list)\n", cfline);
 		return 0; 
 	}
-	if (crit && crit->exdgspec && namematch(xmh_item(hinfo, XMH_DGNAME), crit->exdgspec, crit->exdgspecre)) { 
+	if (crit && crit->exdgspec && namematch(dgname, crit->exdgspec, crit->exdgspecre)) { 
 		traceprintf("Failed '%s' (displaygroup excluded)\n", cfline);
 		return 0; 
 	}
@@ -980,10 +999,11 @@ static int criteriamatch(activealerts_t *alert, criteria_t *crit, criteria_t *ru
 	 * some random system recovered ... not good. So apply
 	 * this check to all messages.
 	 */
-	if (crit && crit->timespec && !timematch(xmh_item(hinfo, XMH_HOLIDAYS), crit->timespec)) {
+	if (crit && ( (crit->timespec && !timematch(xmh_item(hinfo, XMH_HOLIDAYS), crit->timespec)) || 
+		      (crit->extimespec && timematch(xmh_item(hinfo, XMH_HOLIDAYS), crit->extimespec)) ) ) {
 		/* Try again in a minute */
 		if (nexttime) *nexttime = getcurrenttime(NULL) + 60;
-		traceprintf("Failed '%s' (time criteria)\n", cfline);
+		traceprintf("Failed '%s' (time/extime criteria)\n", cfline);
 		if (!printmode) return 0; 
 	}
 
@@ -1126,7 +1146,7 @@ void print_alert_recipients(activealerts_t *alert, strbuffer_t *buf)
 	stoprulefound = 0;
 	while ((recip = next_recipient(alert, &first, NULL, NULL)) != NULL) {
 		int mindur = 0, maxdur = INT_MAX;
-		char *timespec = NULL;
+		char *timespec = NULL; char *extimespec = NULL;
 		int colors = defaultcolors;
 		int i, firstcolor = 1;
 		int recovered = 0, notice = 0;
@@ -1154,10 +1174,15 @@ void print_alert_recipients(activealerts_t *alert, strbuffer_t *buf)
 		if (printrule->criteria && printrule->criteria->maxduration) maxdur = printrule->criteria->maxduration;
 		if (recip->criteria && recip->criteria->maxduration && (recip->criteria->maxduration < maxdur)) maxdur = recip->criteria->maxduration;
 		if (printrule->criteria && printrule->criteria->timespec) timespec = printrule->criteria->timespec;
+		if (printrule->criteria && printrule->criteria->extimespec) extimespec = printrule->criteria->extimespec;
 		if (recip->criteria && recip->criteria->timespec) {
 			if (timespec == NULL) timespec = recip->criteria->timespec;
 			else errprintf("Cannot handle nested timespecs yet\n");
 		}
+		if (recip->criteria && recip->criteria->extimespec) {
+			if (extimespec == NULL) extimespec = recip->criteria->extimespec;
+			else errprintf("Cannot handle nested extimespecs yet\n");
+		}
 
 		if (printrule->criteria && printrule->criteria->colors) colors = (colors & printrule->criteria->colors);
 		if (recip->criteria && recip->criteria->colors) colors = (colors & recip->criteria->colors);
@@ -1203,7 +1228,10 @@ void print_alert_recipients(activealerts_t *alert, strbuffer_t *buf)
 		sprintf(l, "<td align=center>%s</td>", durationstring(recip->interval)); 
 		addtobuffer(buf, l);
 
-		if (timespec) sprintf(l, "<td align=center>%s</td>", timespec); else strcpy(l, "<td align=center>-</td>");
+		if (timespec && extimespec) sprintf(l, "<td align=center>%s, except %s</td>", timespec, extimespec);
+		else if (timespec) sprintf(l, "<td align=center>%s</td>", timespec);
+		else if (extimespec) sprintf(l, "<td align=center>all, except %s</td>", extimespec);
+		else strcpy(l, "<td align=center>-</td>");
 		addtobuffer(buf, l);
 
 		addtobuffer(buf, "<td>");
