@@ -38,6 +38,7 @@ enum xmh_item_t {
 	XMH_DELAYYELLOW,
 	XMH_FLAG_NOINFO,
 	XMH_FLAG_NOTRENDS,
+	XMH_FLAG_NOCLIENT,
 	XMH_FLAG_NODISP,
 	XMH_FLAG_NONONGREEN,
 	XMH_FLAG_NOBB2,
@@ -52,7 +53,7 @@ enum xmh_item_t {
 	XMH_FLAG_LDAPFAILYELLOW,
 	XMH_FLAG_NOCLEAR,
 	XMH_FLAG_HIDEHTTP,
-	XMH_FLAG_PULLDATA,
+	XMH_PULLDATA,
 	XMH_FLAG_MULTIHOMED,
 	XMH_FLAG_SNI,
 	XMH_FLAG_NOSNI,
