@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: misc.c 7040 2012-07-13 16:29:38Z storner $";
+static char rcsid[] = "$Id: misc.c 7355 2014-01-19 21:18:21Z storner $";
 
 #include "config.h"
 
@@ -204,7 +204,7 @@ int argnmatch(char *arg, char *match)
 }
 
 
-char *msg_data(char *msg)
+char *msg_data(char *msg, int stripcr)
 {
 	/* Find the start position of the data following the "status host.test " message */
 	char *result;
@@ -220,6 +220,15 @@ char *msg_data(char *msg)
 	result += strcspn(result, " \t\n");     /* Skip anything until we see a space, TAB or NL */
 	result += strspn(result, " \t");        /* Skip all whitespace */
 
+	if (stripcr) {
+		/* Replace <cr> with blanks */
+		char *cr = result;
+		do {
+			cr = strchr(cr, '\r');
+			if (cr) *cr = ' ';
+		} while (cr);
+	}
+
 	return result;
 }
 
