@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: url.c 7037 2012-07-13 14:44:27Z storner $";
+static char rcsid[] = "$Id: url.c 7632 2015-04-17 15:55:57Z jccleaver $";
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -307,23 +307,26 @@ void parse_url(char *inputurl, urlelem_t *url)
 	p = strchr(startp, ':');
 	if (p) {
 		*p = '\0';
-		if (strncmp(startp, "https", 5) == 0) {
+		if (strncasecmp(startp, "https", 5) == 0) {
 			url->scheme = "https";
 			url->port = 443;
 			if (strlen(startp) > 5) url->schemeopts = strdup(startp+5);
-		} else if (strncmp(startp, "http", 4) == 0) {
+		} else if (strncasecmp(startp, "http", 4) == 0) {
 			url->scheme = "http";
 			url->port = 80;
 			if (strlen(startp) > 4) url->schemeopts = strdup(startp+4);
-		} else if (strcmp(startp, "ftp") == 0) {
+		} else if (strncasecmp(startp, "ftps", 4) == 0) {
+			url->scheme = "ftps";
+			url->port = 990;
+		} else if (strncasecmp(startp, "ftp", 3) == 0) {
 			url->scheme = "ftp";
 			url->port = 21;
-		} else if (strcmp(startp, "ldap") == 0) {
-			url->scheme = "ldap";
-			url->port = 389;
-		} else if (strcmp(startp, "ldaps") == 0) {
+		} else if (strncasecmp(startp, "ldaps", 5) == 0) {
 			url->scheme = "ldaps";
 			url->port = 389; /* ldaps:// URL's are non-standard, and must use port 389+STARTTLS */
+		} else if (strncasecmp(startp, "ldap", 4) == 0) {
+			url->scheme = "ldap";
+			url->port = 389;
 		}
 		else {
 			/* Unknown scheme! */
@@ -533,6 +536,12 @@ char *decode_url(char *testspec, weburl_t *weburl)
 	} else if (strncmp(inp, "httpstatus=", 11) == 0) {
 		weburl->testtype = WEBTEST_STATUS;
 		urlstart = gethttpcolumn(inp+11, &weburl->columnname);
+	} else if (strncmp(inp, "httphead;", 9) == 0) {
+		weburl->testtype = WEBTEST_HEAD;
+		urlstart = strchr(inp, ';') + 1;
+	} else if (strncmp(inp, "httphead=", 9) == 0) {
+		weburl->testtype = WEBTEST_HEAD;
+		urlstart = gethttpcolumn(inp+9, &weburl->columnname);
 	} else if (strncmp(inp, "http=", 5) == 0) {
 		/* Plain URL test, but in separate column */
 		weburl->testtype = WEBTEST_PLAIN;
@@ -545,6 +554,7 @@ char *decode_url(char *testspec, weburl_t *weburl)
 
 	switch (weburl->testtype) {
 	  case WEBTEST_PLAIN:
+	  case WEBTEST_HEAD:
 		  break;
 
 	  case WEBTEST_CONT:
