@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: showgraph.c 7128 2012-07-28 06:29:24Z storner $";
+static char rcsid[] = "$Id: showgraph.c 7442 2014-02-23 09:32:44Z storner $";
 
 #include <limits.h>
 #include <stdio.h>
@@ -158,7 +158,13 @@ void request_cacheflush(char *hostname)
 			do {
 				n = sendto(ctlsocket, bufp, bytesleft, 0, (struct sockaddr *)&myaddr, myaddrsz);
 				if (n == -1) {
-					if (errno != EAGAIN) {
+					if (errno == EDESTADDRREQ) {
+						/* Probably a left-over rrdctl file, ignore it */
+					}
+					else if (errno == EAGAIN) {
+						/* Harmless */
+					}
+					else {
 						errprintf("Sendto failed: %s\n", strerror(errno));
 					}
 
@@ -664,8 +670,8 @@ void graph_link(FILE *output, char *uri, char *grtype, time_t seconds)
 	  case ACT_MENU:
 		fprintf(output, "  <td align=\"left\"><img src=\"%s&amp;action=view&amp;graph=%s\" alt=\"%s graph\"></td>\n",
 			uri, grtype_s, grtype_s);
-		fprintf(output, "  <td align=\"left\" valign=\"top\"> <a href=\"%s&amp;graph=%s&amp;action=selzoom&amp;color=%s\"> <img src=\"%s/zoom.gif\" border=0 alt=\"Zoom graph\" style='padding: 3px'> </a> </td>\n",
-			uri, grtype_s, colorname(bgcolor), getenv("XYMONSKIN"));
+		fprintf(output, "  <td align=\"left\" valign=\"top\"> <a href=\"%s&amp;graph=%s&amp;action=selzoom&amp;color=%s\"> <img src=\"%s/zoom.%s\" border=0 alt=\"Zoom graph\" style='padding: 3px'> </a> </td>\n",
+			uri, grtype_s, colorname(bgcolor), xgetenv("XYMONSKIN"), xgetenv("IMAGEFILETYPE"));
 		break;
 
 	  case ACT_SELZOOM:
@@ -712,6 +718,11 @@ char *build_selfURI(void)
 		addtobuffer(result, "&amp;service=");
 		addtobuffer(result, urlencode(service));
 	}
+	if (haveupperlimit) {
+		snprintf(numbuf, sizeof(numbuf)-1, "%f", upperlimit);
+		addtobuffer(result, "&amp;upper=");
+		addtobuffer(result, urlencode(numbuf));
+	}
 	if (graphheight) {
 		snprintf(numbuf, sizeof(numbuf)-1, "%d", graphheight); 
 		addtobuffer(result, "&amp;graph_height="); 
