@@ -12,7 +12,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: svcstatus-info.c 7076 2012-07-15 10:49:22Z storner $";
+static char rcsid[] = "$Id: svcstatus-info.c 7644 2015-05-01 02:44:02Z jccleaver $";
 
 #include <limits.h>
 #include <stdio.h>
@@ -94,7 +94,7 @@ static int fetch_status(char *hostname)
 		eol = strchr(walk, '\n'); if (eol) *eol = '\0';
 
 		tok = gettok(walk, "|");
-		if ( tok && (strcmp(tok, xgetenv("INFOCOLUMN")) != 0) && (strcmp(tok, xgetenv("TRENDSCOLUMN")) != 0) ) {
+		if ( tok && (strcmp(tok, xgetenv("INFOCOLUMN")) != 0) && (strcmp(tok, xgetenv("TRENDSCOLUMN")) != 0) && (strcmp(tok, xgetenv("CLIENTCOLUMN")) != 0) ) {
 			tnames[testcount].name = strdup(tok); tok = gettok(NULL, "|"); 
 			if (tok) { tnames[testcount].color = parse_color(tok); tok = gettok(NULL, "|"); }
 			if (tok) { tnames[testcount].distime = atol(tok); tok = gettok(NULL, "|"); }
