@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: xymond_client.c 7252 2013-08-05 11:54:11Z storner $";
+static char rcsid[] = "$Id: xymond_client.c 7636 2015-04-19 00:36:43Z jccleaver $";
 
 #include <stdio.h>
 #include <string.h>
@@ -50,6 +50,7 @@ int sendclearsvcs = 1;
 int localmode     = 0;
 int unknownclientosok = 0;
 int noreportcolor = COL_CLEAR;
+int separate_uptime_status = 0;
 
 int usebackfeedqueue = 0;
 
@@ -216,7 +217,7 @@ char *nextsection(char *clientdata, char **name)
 		defsecthead = NULL;
 	}
 
-	return nextsection_r(clientdata, name, &current, (void **)&defsecthead);
+	return nextsection_r(clientdata, name, &current, (void *)defsecthead);
 }
 
 
@@ -315,10 +316,10 @@ void unix_cpu_report(char *hostname, char *clientclass, enum ostype_t os,
 	long uptimesecs = -1;
 	char myupstr[100];
 
-	int cpucolor = COL_GREEN;
+	int cpucolor = COL_GREEN, upstatuscolor = COL_GREEN;
 
 	char msgline[4096];
-	strbuffer_t *upmsg;
+	strbuffer_t *cpumsg, *upmsg = NULL;
 
 	if (!want_msgtype(hinfo, MSG_CPU)) return;
 
@@ -421,27 +422,45 @@ void unix_cpu_report(char *hostname, char *clientclass, enum ostype_t os,
 
 	get_cpu_thresholds(hinfo, clientclass, &loadyellow, &loadred, &recentlimit, &ancientlimit, &uptimecolor, &maxclockdiff, &clockdiffcolor);
 
-	upmsg = newstrbuffer(0);
+	cpumsg = newstrbuffer(0);
 
 	if (load5 > loadred) {
 		cpucolor = COL_RED;
-		addtobuffer(upmsg, "&red Load is CRITICAL\n");
+		addtobuffer(cpumsg, "&red Load is CRITICAL\n");
 	}
 	else if (load5 > loadyellow) {
 		cpucolor = COL_YELLOW;
-		addtobuffer(upmsg, "&yellow Load is HIGH\n");
+		addtobuffer(cpumsg, "&yellow Load is HIGH\n");
 	}
 
-	if ((uptimesecs != -1) && (recentlimit != -1) && (uptimesecs < recentlimit)) {
-		if (cpucolor != COL_RED) cpucolor = uptimecolor;
-		sprintf(msgline, "&%s Machine recently rebooted\n", colorname(uptimecolor));
-		addtobuffer(upmsg, msgline);
+	if (separate_uptime_status) {
+		upmsg = newstrbuffer(0);
+
+		if ((uptimesecs != -1) && (recentlimit != -1) && (uptimesecs < recentlimit)) {
+			upstatuscolor = uptimecolor;
+			sprintf(msgline, "&%s Machine recently rebooted\n", colorname(uptimecolor));
+			addtobuffer(upmsg, msgline);
+		}
+		if ((uptimesecs != -1) && (ancientlimit != -1) && (uptimesecs > ancientlimit)) {
+			upstatuscolor = uptimecolor;
+			sprintf(msgline, "&%s Machine has been up more than %d days\n", 
+				colorname(uptimecolor), (ancientlimit / 86400));
+			addtobuffer(upmsg, msgline);
+		}
+
 	}
-	if ((uptimesecs != -1) && (ancientlimit != -1) && (uptimesecs > ancientlimit)) {
-		if (cpucolor != COL_RED) cpucolor = uptimecolor;
-		sprintf(msgline, "&%s Machine has been up more than %d days\n", 
-			colorname(uptimecolor), (ancientlimit / 86400));
-		addtobuffer(upmsg, msgline);
+	else {
+		if ((uptimesecs != -1) && (recentlimit != -1) && (uptimesecs < recentlimit)) {
+			if (cpucolor != COL_RED) cpucolor = uptimecolor;
+			sprintf(msgline, "&%s Machine recently rebooted\n", colorname(uptimecolor));
+			addtobuffer(cpumsg, msgline);
+		}
+		if ((uptimesecs != -1) && (ancientlimit != -1) && (uptimesecs > ancientlimit)) {
+			if (cpucolor != COL_RED) cpucolor = uptimecolor;
+			sprintf(msgline, "&%s Machine has been up more than %d days\n", 
+				colorname(uptimecolor), (ancientlimit / 86400));
+			addtobuffer(cpumsg, msgline);
+		}
 	}
 
 	if (clockstr) {
@@ -472,11 +491,11 @@ void unix_cpu_report(char *hostname, char *clientclass, enum ostype_t os,
 				if (cpucolor != COL_RED) cpucolor = clockdiffcolor;
 				sprintf(msgline, "&%s System clock is %ld seconds off (max %ld)\n",
 					colorname(clockdiffcolor), (long) clockdiff.tv_sec, (long) maxclockdiff);
-				addtobuffer(upmsg, msgline);
+				addtobuffer(cpumsg, msgline);
 			}
 			else {
 				sprintf(msgline, "System clock is %ld seconds off\n", (long) clockdiff.tv_sec);
-				addtobuffer(upmsg, msgline);
+				addtobuffer(cpumsg, msgline);
 			}
 		}
 	}
@@ -490,8 +509,8 @@ void unix_cpu_report(char *hostname, char *clientclass, enum ostype_t os,
 		(psstr ? linecount(psstr)-1 : pscount), 
 		loadresult);
 	addtostatus(msgline);
-	if (STRBUFLEN(upmsg)) {
-		addtostrstatus(upmsg);
+	if (STRBUFLEN(cpumsg)) {
+		addtostrstatus(cpumsg);
 		addtostatus("\n");
 	}
 	if (topstr) {
@@ -502,7 +521,21 @@ void unix_cpu_report(char *hostname, char *clientclass, enum ostype_t os,
 	if (fromline && !localmode) addtostatus(fromline);
 	finish_status();
 
-	freestrbuffer(upmsg);
+	if (separate_uptime_status) {
+		init_status(upstatuscolor);
+		sprintf(msgline, "status %s.uptime %s %s Uptime %s\n",
+			commafy(hostname), colorname(upstatuscolor), 
+			(timestr ? timestr : "<no timestamp data>"),
+			((upstatuscolor == COL_GREEN) ? "OK" : "Not OK"));
+		addtostatus(msgline);
+		sprintf(msgline, "\nSystem has been %s\n", myupstr);
+		addtostatus(msgline);
+		if (fromline && !localmode) addtostatus(fromline);
+		finish_status();
+	}
+
+	freestrbuffer(cpumsg);
+	if (upmsg) freestrbuffer(upmsg);
 }
 
 
@@ -1021,10 +1054,8 @@ void unix_procs_report(char *hostname, char *clientclass, enum ostype_t os,
 	}
 	if (eol) *eol = '\n';
 
-	if (debug) {
-		if (cmdofs >= 0) dbgprintf("Host %s: Found ps command line at offset %d\n", hostname, cmdofs);
-		else dbgprintf("Host %s: None of the headings found\n", hostname);
-	}
+	if (cmdofs >= 0) { dbgprintf("Host %s: Found ps command line at offset %d\n", hostname, cmdofs); }
+	else { dbgprintf("Host %s: None of the headings found\n", hostname); }
 
 	pchecks = clear_process_counts(hinfo, clientclass);
 
@@ -1119,41 +1150,7 @@ void unix_procs_report(char *hostname, char *clientclass, enum ostype_t os,
 	}
 
 	/* And the full ps output for those who want it */
-	if (pslistinprocs) {
-		/*
-		 * NB: Process listings may contain HTML special characters.
-		 *     We must encode these for HTML, cf.
-		 *     http://www.w3.org/TR/html4/charset.html#h-5.3.2
-		 */
-		char *inp, *tagpos;
-
-		inp = psstr;
-		do {
-			tagpos = inp + strcspn(inp, "<>&\"");
-			switch (*tagpos) {
-			  case '<':
-				*tagpos = '\0'; addtostatus(inp); addtostatus("&lt;"); *tagpos = '<';
-				inp = tagpos + 1;
-				break;
-			  case '>':
-				*tagpos = '\0'; addtostatus(inp); addtostatus("&gt;"); *tagpos = '>';
-				inp = tagpos + 1;
-				break;
-			  case '&':
-				*tagpos = '\0'; addtostatus(inp); addtostatus("&amp;"); *tagpos = '&';
-				inp = tagpos + 1;
-				break;
-			  case '\"':
-				*tagpos = '\0'; addtostatus(inp); addtostatus("&quot;"); *tagpos = '\"';
-				inp = tagpos + 1;
-				break;
-			  default:
-				/* We're done */
-				addtostatus(inp); inp = NULL;
-				break;
-			}
-		} while (inp && *inp);
-	}
+	if (pslistinprocs) addtostatus(prehtmlquoted(psstr));
 
 	if (fromline && !localmode) addtostatus(fromline);
 	finish_status();
@@ -1256,9 +1253,7 @@ void msgs_report(char *hostname, char *clientclass, enum ostype_t os,
 					swalk->sname+5);
 			}
 			addtobuffer(yellowdata, msgline);
-			addtobuffer(yellowdata, "<pre>\n");
 			addtostrbuffer(yellowdata, logsummary);
-			addtobuffer(yellowdata, "</pre>\n");
 			break;
 
 		  case COL_RED:
@@ -1271,9 +1266,7 @@ void msgs_report(char *hostname, char *clientclass, enum ostype_t os,
 					swalk->sname+5);
 			}
 			addtobuffer(reddata, msgline);
-			addtobuffer(yellowdata, "<pre>\n");
 			addtostrbuffer(reddata, logsummary);
-			addtobuffer(yellowdata, "</pre>\n");
 			break;
 		}
 
@@ -1328,9 +1321,7 @@ void msgs_report(char *hostname, char *clientclass, enum ostype_t os,
 				swalk->sname+5);
 		}
 		addtostatus(msgline);
-		addtobuffer(yellowdata, "<pre>\n");
-		addtostatus(swalk->sdata);
-		addtobuffer(yellowdata, "</pre>\n");
+		addtostatus(prehtmlquoted(swalk->sdata));
 		do { swalk=swalk->next; } while (swalk && strncmp(swalk->sname, "msgs:", 5));
 	}
 
@@ -1550,7 +1541,50 @@ void linecount_report(char *hostname, char *clientclass, enum ostype_t os,
 				countstr = (id ? strtok(NULL, "\n") : NULL);
 				if (id && countstr) {
 					countstr += strspn(countstr, "\t ");
-					sprintf(msgline, "%s#%s:%s\n", nocolon(fn), id, countstr);
+					snprintf(msgline, sizeof(msgline), "%s#%s:%s\n", nocolon(fn), id, countstr);
+					addtobuffer(countdata, msgline);
+				}
+
+				boln = (eoln ? eoln + 1 : NULL);
+			}
+		}
+	}
+
+	if (anydata) combo_add(countdata);
+	clearstrbuffer(countdata);
+}
+
+
+void deltacount_report(char *hostname, char *clientclass, enum ostype_t os,
+			void *hinfo, char *fromline, char *timestr)
+{
+	static strbuffer_t *countdata = NULL;
+	sectlist_t *swalk;
+	char msgline[PATH_MAX];
+	int anydata = 0;
+
+	if (!countdata) countdata = newstrbuffer(0);
+
+	sprintf(msgline, "data %s.deltacounts\n", commafy(hostname));
+	addtobuffer(countdata, msgline);
+
+	for (swalk = defsecthead; (swalk); swalk = swalk->next) {
+		if (strncmp(swalk->sname, "deltacount:", 10) == 0) {
+			char *fn, *boln, *eoln, *id, *countstr;
+
+			anydata = 1;
+
+			fn = strchr(swalk->sname, ':'); fn += 1 + strspn(fn+1, "\t ");
+
+			boln = swalk->sdata;
+			while (boln) {
+				eoln = strchr(boln, '\n');
+
+				id = strtok(boln, ":");
+				countstr = (id ? strtok(NULL, "\n") : NULL);
+				if (id && countstr) {
+					countstr += strspn(countstr, "\t ");
+					snprintf(msgline, sizeof(msgline), "%s#%s:%s\n", nocolon(fn), id, countstr);
 					addtobuffer(countdata, msgline);
 				}
 
@@ -1612,11 +1646,12 @@ void unix_vmstat_report(char *hostname, char *clientclass, enum ostype_t os,
 	p = strrchr(vmstatstr, '\n');
 	if (!p) return;  /* No NL in vmstat output ? Unlikely. */
 
-	msg = newstrbuffer(0);
 	if (strlen(p) == 1) {
 		/* Go back to the previous line */
 		do { p--; } while ((p > vmstatstr) && (*p != '\n'));
 	}
+	if (strcmp(p+1,"Terminated") == 0) return;	/* vmstat process was killed early - ignore data */
+	msg = newstrbuffer(0);
 	sprintf(msgline, "data %s.vmstat\n%s\n", commafy(hostname), osname(os));
 	addtobuffer(msg, msgline);
 	addtobuffer(msg, p+1);
@@ -2058,6 +2093,9 @@ int main(int argc, char *argv[])
 			char *p = strchr(argv[argi], '=');
 			noreportcolor = parse_color(p+1);
 		}
+		else if (strcmp(argv[argi], "--uptime-status") == 0) {
+			separate_uptime_status = 1;
+		}
 		else if (argnmatch(argv[argi], "--config=")) {
 			char *lp = strchr(argv[argi], '=');
 			configfn = strdup(lp+1);
