@@ -8,7 +8,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: xymonproxy.c 7368 2014-01-26 11:08:47Z storner $";
+static char rcsid[] = "$Id: xymonproxy.c 7635 2015-04-19 00:26:20Z jccleaver $";
 
 #include "config.h"
 
@@ -266,6 +266,7 @@ int main(int argc, char *argv[])
 				errprintf("Invalid listen address %s\n", locaddr);
 				return 1;
 			}
+			if (p) *p = ':';
 		}
 		else if (argnmatch(argv[opt], "--server=") || argnmatch(argv[opt], "--bbdisplay=")) {
 			char *ips, *ip1;
@@ -492,7 +493,7 @@ int main(int argc, char *argv[])
 			}
 
 			p = stentry->buf;
-			p += sprintf(p, "combo\nstatus %s green %s Proxy up %s\n\nxymonproxy for Xymon version %s\n\nProxy statistics\n\nIncoming messages        : %10lu (%lu msgs/second)\nOutbound messages        : %10lu\n\nIncoming message distribution\n- Combo messages         : %10lu\n- Status messages        : %10lu\n  Messages merged        : %10lu\n  Resulting combos       : %10lu\n- Other messages         : %10lu\n\nProxy ressources\n- Connection table size  : %10d\n- Buffer space           : %10lu kByte\n",
+			p += sprintf(p, "combo\nstatus+11 %s green %s - xymon proxy up: %s\n\nxymonproxy for Xymon version %s\n\nProxy statistics\n\nIncoming messages        : %10lu (%lu msgs/second)\nOutbound messages        : %10lu\n\nIncoming message distribution\n- Combo messages         : %10lu\n- Status messages        : %10lu\n  Messages merged        : %10lu\n  Resulting combos       : %10lu\n- Other messages         : %10lu\n\nProxy ressources\n- Connection table size  : %10d\n- Buffer space           : %10lu kByte\n",
 				proxyname, timestamp, runtime_s, VERSION,
 				msgs_total, (msgs_total - msgs_total_last) / (now - laststatus),
 				msgs_delivered,
@@ -852,7 +853,7 @@ int main(int argc, char *argv[])
 					cwalk->ssocket = -1;
 				}
 				cwalk->arrival.tv_sec = cwalk->arrival.tv_nsec = 0;
-				cwalk->bufp = cwalk->bufp; 
+				cwalk->bufp = cwalk->buf; 
 				cwalk->buflen = 0;
 				memset(cwalk->buf, 0, cwalk->bufsize);
 				memset(&cwalk->caddr, 0, sizeof(cwalk->caddr));
@@ -963,7 +964,7 @@ int main(int argc, char *argv[])
 		}
 
 		if (combining) {
-			selecttmo.tv_sec = 0; selecttmo.tv_usec = COMBO_DELAY;
+			selecttmo.tv_sec = 0; selecttmo.tv_usec = COMBO_DELAY / 1000;
 		}
 		else {
 			selecttmo.tv_sec = 1; selecttmo.tv_usec = 0;
